// -*- verilog -*-
//
//  USRP - Universal Software Radio Peripheral
//
//  Copyright (C) 2003 Matt Ettus
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin Street, Boston, MA  02110-1301  USA
//


// This module transforms an input stream of 16-bit words into an
// output stream of packed n-bit words where n is 16, 8, 4, 2, or 1.
//
// The bit field from the input words is selectable with the "shift"
// port, which tells how far to shift the input right before extracting
// n LSBs:  out_fragment = (in>>shift) & ((1<<n)-1).  The sequence
// of out_fragments is packed bigendian into the output 16-bit payload.
//
// Handshaking is with a synchronous dataflow convention in which
// in_srdy indicates that an input word is available and in_drdy indicates
// that the bit_pack module can accept an input word.  Thus, on any given
// cycle, a word is transferred over the input interface if and only if
// both in_srdy and in_drdy are asserted.  Similarly, out_srdy indicates
// that the bit_pack module has an output word available and out_drdy
// indicates that the downstream entity can accept an output word;
// an output word is transferred if and only if both are asserted.
//
// Copyright 2007 Peter Monta

module bit_pack
  ( input clk,
    input reset,
    input [15:0] in,
    input in_srdy,
    output in_drdy,
    input [4:0] n,
    input [3:0] shift,
    output [15:0] out,
    output out_srdy,
    input out_drdy
  );

  reg [15:0] out1;
  reg [4:0] b1;

  wire in_en = in_srdy && in_drdy;
  wire out_en = out_srdy && out_drdy;

  wire [4:0] b = in_en ? (b1 + n) : b1;

  assign in_drdy = (b1<5'd16) && out_drdy;
  assign out_srdy = (b>=5'd16);

  reg [15:0] mask;

  assign out = (out1<<n) | ((in>>shift) & mask);

  always @*
    case (n)
      5'd1: mask = 16'b0000000000000001;
      5'd2: mask = 16'b0000000000000011;
      5'd4: mask = 16'b0000000000001111;
      5'd8: mask = 16'b0000000011111111;
      5'd16: mask = 16'b1111111111111111;
      default: mask = 16'd0;
    endcase

  always @(posedge clk)
    if (reset) begin
      out1 <= #1 0;
      b1 <= #1 0;
    end else
      if (in_en) begin
        out1 <= #1 out;
        b1 <= #1 out_en ? 0 : b;
      end

endmodule // bit_pack
